"""
Functions for progress bars

"""

from tqdm import tqdm

def iter_progress(iterable, label=''):
    return tqdm(iterable, desc=label, leave=True)

def enum_progress(iterable, label=''):
    return enumerate(tqdm(iterable, desc=label, leave=True))



"""

This file generates the index.html file, using Bootstrap

"""

import dominate
import os

from .common import navbar
from .utils import AbedHTMLTypes, copy_data_file
from ..conf import settings
from ..tasks import init_tasks
from ..utils import info

tags = dominate.tags

def create_index_html(task_dict):
    idx = generate_index_html(task_dict)
    write_index_html(idx)

def get_status_perc(task_dict):
    all_tasks = init_tasks()
    done = len(all_tasks) - len(task_dict)
    perc = float(done)/float(len(all_tasks)) * 100.0
    return int(round(perc))

def generate_index_html(task_dict):
    doc = dominate.document(title='ABED Main page')

    with doc.head:
        # meta tags
        tags.meta(charset='utf-8')
        tags.meta(http_equiv='X-UA-Compatible', content='IE=edge')
        tags.meta(name='viewport',
                content='width=device-width, initial-scale=1')
        tags.meta(name='description', content='')
        tags.meta(name='author', content='')
        #tags.link(rel='icon', href=copy_data_file('ABED/images/favicon.ico'))

        # JQuery & Bootstrap JS
        tags.comment('Bootstrap core JavaScript')
        tags.script(src=copy_data_file('jquery/1.11.3/jquery.min.js'))
        tags.script(
                src=copy_data_file('bootstrap-3.3.5-dist/js/bootstrap.min.js'))

        # Bootstrap core CSS
        tags.link(rel='stylesheet', href=copy_data_file(
            'bootstrap-3.3.5-dist/css/bootstrap.min.css'))

        # ABED CSS
        tags.link(rel='stylesheet', href=copy_data_file('abed/css/abed.css'))

    with doc:
        navbar(active=AbedHTMLTypes.INDEX)
        with tags.div(_class='container'):
            with tags.div(_class='abed-title'):
                tags.h1('ABED Results Overview for project %s' % 
                        settings.PROJECT_NAME)
            tags.p('Percentage of tasks completed:')
            with tags.div(_class='progress'):
                perc = str(get_status_perc(task_dict))
                tags.div('%s%%' % perc, _class='progress-bar', 
                        role='progressbar', aria_valuenow=perc, 
                        aria_minvalue='0', aria_maxvalue='100',
                        style='width: %s%%;' % perc)
        tags.comment('/.container')

    return str(doc)

def write_index_html(doc):
    fname = '%s%s%s%s%s' % (settings.OUTPUT_DIR, os.sep, 'html', os.sep, 
            'index.html')
    with open(fname, 'w') as fid:
        fid.write(doc)
    info("Created output file: %s" % fname)



import sys

from .conf import settings
from .help import get_help, get_command_help
from .models import Abed
from .utils import info, error

def parse_command(command):
    choices = list(Abed.commands)
    choices.append('help')
    if not command in choices:
        print("abed: '%s' is not a valid command. See 'abed help'" % command)
        print("")
        raise SystemExit
    return command

def parse_arguments():
    cmdargs = sys.argv[1:]
    if len(cmdargs) == 0:
        print(get_help())
        raise SystemExit

    args = {'skip_cache': False, 'cmd': None, 'topic': None}
    idx = 0
    args['cmd'] = parse_command(cmdargs[idx])
    idx += 1
    if args['cmd'] == 'help' and len(cmdargs) > idx:
        args['topic'] = parse_command(cmdargs[idx])
    elif args['cmd'] == 'parse_results' and len(cmdargs) > idx:
        if cmdargs[idx] in ["-s", "--skip-cache"]:
            args['skip_cache'] = True
        else:
            error("Unknown command line argument: %s." % cmdargs[idx])
            error("See 'abed help parse_results' for help.")
            raise SystemExit
    elif len(cmdargs) > idx:
            error("Unknown command line argument: %s." % cmdargs[idx])
            error("See 'abed help' for help.")
            raise SystemExit

    return args

def main():
    args = parse_arguments()
    if args['cmd'] == 'help':
        if args['topic'] is None:
            print(get_help())
            raise SystemExit
        else:
            print(get_command_help(args['topic']))
            raise SystemExit

    skip_init = False
    if args['cmd'] == 'reload_tasks':
        skip_init = True

    if settings is None:
        if not args['cmd'] == 'init':
            error("No ABED configuration file found in this directory. "
                    "Run 'abed init' to initialize one. Exiting.")
            raise SystemExit
        skip_init = True
    abed = Abed(skip_init=skip_init, skip_cache=args['skip_cache'])

    try:
        getattr(abed, args['cmd'])()
    except KeyboardInterrupt:
        pass



"""
Constants used in ABED

"""

#: Default filename for the abed config file.
CONFIG_FILENAME = 'abed_conf.py'

#: Default dataset directory name
DATASET_DIRNAME = 'datasets'

#: Default executables directory name
EXECS_DIRNAME = 'execs'

#: Default tasksfile filename
TASKS_FILENAME = 'abed_tasks.txt'

#: Default auto filename
AUTO_FILENAME = 'abed_auto.txt'



"""
Utility functions for generating html. Includes functions for copying necessary 
assets.
"""

import os
import shutil

import abed

from ..conf import settings
from ..utils import mkdir

class AbedHTMLTypes:
    INDEX = ('Home', 'index.html')
    METRIC_TABLES = ('Metric Tables', 'metric_tables.html')
    SCALAR_TABLES = ('Scalar Tables', 'scalar_tables.html')
    PROFILE_PLOTS = ('Profile Plots', 'profile_plots.html')
    RANK_GRAPHS = ('Rank Graphs', 'rank_graphs.html')
    RANK_TESTS = ('Rank Tests', 'rank_tests.html')
    HISTOGRAMS = ('Histograms', 'histograms.html')
    types = [INDEX, METRIC_TABLES, SCALAR_TABLES, PROFILE_PLOTS, RANK_GRAPHS,
            RANK_TESTS, HISTOGRAMS]

def get_data_path(filepath):
    packagedir = abed.__path__[0]
    dirname = os.path.join(os.path.dirname(packagedir), 'share', 'data')
    fullname = os.path.join(dirname, filepath)
    fullpath = os.path.abspath(fullname)
    return fullpath

def copy_data_file(filepath):
    src = get_data_path(filepath)
    datapath = os.path.join('assets', filepath)
    dest = os.path.join(settings.OUTPUT_DIR, 'html', datapath)
    destdir = os.path.dirname(dest)
    mkdir(destdir)
    shutil.copy(src, dest)
    return datapath

def copy_auxiliary_files():
    copy_data_file('DataTables-1.10.7/media/images/favicon.ico')
    copy_data_file('DataTables-1.10.7/media/images/sort_both.png')
    copy_data_file('DataTables-1.10.7/media/images/sort_asc.png')
    copy_data_file('DataTables-1.10.7/media/images/sort_asc_disabled.png')
    copy_data_file('DataTables-1.10.7/media/images/sort_desc.png')
    copy_data_file('DataTables-1.10.7/media/images/sort_desc_disabled.png')
    copy_data_file('DataTables-1.10.7/media/images/Sorting icons.psd')





"""
Various utility functions used throughout abed

"""

from __future__ import print_function

import errno
import os
import sys

basename = os.path.basename
splitext = os.path.splitext


def wrap_text(text, max_length=120):
    """
    Wraps the words into lines of a fixed length for prettier printing.
    """

    words = []
    for part in text.split('\n'):
        words.extend(part.split(' '))
        words.append('\n')
    sentences = []
    current_length = 0
    sentence = ''
    for word in words:
        if word == '\n':
            sentences.append(sentence)
            sentence = ''
            current_length = 0
            continue
        if (current_length + len(word) + 1 <= max_length):
            current_length += len(word) + 1
            sentence += word + ' '
        else:
            current_length = len(word) + 1
            sentences.append(sentence)
            sentence = word + ' '
    return '\n'.join(sentences)


def info(txt):
    message = '%s' % txt
    print(message)


def error(txt):
    message = 'Error: %s' % (txt)
    print(message, file=sys.stderr)


def warning(txt):
    message = 'Warning: %s' % (txt)
    print(message, file=sys.stderr)


def mkdir(path):
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise

def hash_from_filename(filename):
    bname = basename(filename)
    exts = splitext(bname)
    start = exts[0]
    hsh = int(start)
    return hsh

def clean_str(string):
    return '_'.join(string.split(' ')).lower()

def touch(fname, times=None):
    with open(fname, 'a'):
        os.utime(fname, times)



import dominate
import os
import json

from .common import navbar
from .utils import AbedHTMLTypes, copy_data_file
from ..conf import settings
from ..utils import info, clean_str
from ..results.models import AbedTableTypes

tags = dominate.tags

def create_rank_graphs_html(tables):
    html = generate_graphs_html(tables)
    write_tables_html(html)

def get_table_id(table):
    values = []
    values.append(clean_str(table.target))
    values.append(clean_str(table.type))
    if table.is_metric:
        if settings.TYPE == 'ASSESS':
            values.append(clean_str(table.metricname))
        elif settings.TYPE == 'CV_TT':
            values.append(clean_str(table.trainmetricname))
            values.append(clean_str(table.testmetricname))
    if table.is_summary:
        values.append('summary')
    return '_'.join(values)

def generate_graphs_html(tables):
    doc = dominate.document(title='ABED Rank Graphs')

    with doc.head:
        # meta tags
        tags.meta(charset='utf-8')
        tags.meta(http_equiv='X-UA-Compatible', content='IE=edge')
        tags.meta(name='viewport',
                content='width=device-width, initial-scale=1')
        tags.meta(name='description', content='Rank graphs for ABED results')
        tags.meta(name='author', content='Gertjan van den Burg')

        # JQuery & Bootstrap JS
        tags.script(type='text/javascript',
                src=copy_data_file('jquery/1.11.3/jquery.min.js'))
        tags.script(type='text/javascript',
                src=copy_data_file('bootstrap-3.3.5-dist/js/bootstrap.min.js'))
        # Bootstrap core CSS
        tags.link(rel='stylesheet', href=copy_data_file(
            'bootstrap-3.3.5-dist/css/bootstrap.min.css'))

        # D3 and Labella
        tags.script(type='text/javascript',
                src=copy_data_file('d3/d3.min.js'))
        tags.script(type='text/javascript',
                src=copy_data_file('d3kit/d3kit.min.js'))
        tags.script(type='text/javascript',
                src=copy_data_file('labella/labella.min.js'))
        tags.script(type='text/javascript',
                src=copy_data_file('d3kit-timeline/d3kit-timeline.min.js'))

        # ABED CSS & JS
        tags.link(rel='stylesheet', href=copy_data_file('abed/css/abed.css'))
        tags.script(type='text/javascript',
                src=copy_data_file('abed/js/abed_rankgraphs.js'))

    tables = [t for t in tables if t.is_summary and t.type ==
            AbedTableTypes.RANKS]
    labels = sorted(set([t.target for t in tables]))
    lbl_table = {}
    for lbl in labels:
        lbl_table[lbl] = [t for t in tables if t.target == lbl]

    with doc:
        with tags.div(id='header'):
            navbar(active=AbedHTMLTypes.RANK_GRAPHS)
        with tags.div(id='content'):
            with tags.div(_class='container'):
                for lbl in sorted(lbl_table.keys()):
                    with tags.div(_class="panel panel-default"):
                        with tags.div(_class="panel-heading"):
                            tags.h3('Target: %s' % lbl, _class="panel-title")
                        with tags.div(_class="panel-body"):
                            for table in lbl_table[lbl]:
                                tabid = get_table_id(table)
                                if table.is_metric:
                                    if settings.TYPE == 'ASSESS':
                                        tags.p("Metric: %s" %
                                                table.metricname)
                                    elif settings.TYPE == 'CV_TT':
                                        with tags.div(_class="col-xs-6"):
                                            tags.b("Train Metric on %s: %s" %
                                                    (settings.YTRAIN_LABEL,
                                                        table.trainmetricname))
                                        with tags.div(_class="col-xs-6"):
                                            tags.b("Test Metric on %s: %s" %
                                                    (lbl,
                                                        table.testmetricname))
                                with tags.div(id=tabid, _class='AbedRanks'):
                                    pass
                                tags.hr()
                                write_table_json(table, tabid)
            tags.comment('./container')
        with tags.div(id='footer'):
            pass

    return str(doc)

def write_table_json(table, tabid):
    data = []
    averages = next((row for _id, row in table if _id == 'Average'), None)
    headers = table.headers[1:]
    for hdr, avg in zip(headers, averages):
        data.append({'name': hdr, 'time': float(avg)})
    fname = os.path.join(settings.OUTPUT_DIR, 'html', tabid + '.json')
    with open(fname, 'w') as fid:
        fid.write(json.dumps(data))
    info("Created output file: %s" % fname)

def write_tables_html(doc):
    fname = os.path.join(settings.OUTPUT_DIR, 'html',
            AbedHTMLTypes.RANK_GRAPHS[-1])
    with open(fname, 'w') as fid:
        fid.write(doc)
    info('Created output file: %s' % fname)



"""Custom Abed Exceptions

These are all subclasses of the Exception class.

"""


class AbedPBSException(Exception):
    pass

class AbedPBSMultipleException(Exception):
    pass

class AbedHashCollissionException(Exception):
    pass

class AbedNonstandardMetricDirection(Exception):
    pass

class AbedDatasetdirNotFoundException(Exception):
    pass

class AbedMethoddirNotFoundException(Exception):
    pass

class AbedExperimentTypeException(Exception):
    pass



"""
Functions for working with PBS files.

Note: this pbs file depends on the `timeout` command to be available. In some 
cases (such as on LISA) it must be loaded separately through a module. This must 
be in the configuration file then.

"""

import time

from .conf import settings

def sec2str(seconds):
    hours = seconds // 3600
    minutes = (seconds - 3600*hours) // 60
    secs = (seconds % 60) // 1
    return '%02i:%02i:%02i' % (hours, minutes, secs)

def generate_pbs_text():
    txt = []

    # create pbs line
    pbs_line = '#PBS -lnodes=%i' % settings.PBS_NODES
    if not settings.PBS_CPUTYPE is None:
        pbs_line += ':%s' % settings.PBS_CPUTYPE
    if not settings.PBS_CORETYPE is None:
        pbs_line += ':%s' % settings.PBS_CORETYPE
    if not settings.PBS_PPN is None:
        if isinstance(settings.PBS_PPN, int):
            pbs_line += ':ppn=%i' % settings.PBS_PPN
        else:
            pbs_line += ':%s' % settings.PPN
    pbs_line += ' -lwalltime=%s' % sec2str(settings.PBS_WALLTIME * 60)
    txt.append(pbs_line)
    txt.append('')

    # export variables (before loading modules!)
    for export in settings.PBS_EXPORTS:
        txt.append('export %s' % export)
    txt.append('')

    # load modules
    for module in settings.PBS_MODULES:
        txt.append('module load %s' % module)
    txt.append('')

    # current directory variable
    txt.append('CURRENT=%s/releases/current' % settings.REMOTE_DIR)
    txt.append('')

    # result dir
    txt.append('mkdir -p ${CURRENT}/results')
    txt.append('mkdir -p ${TMPDIR}/results')
    txt.append('')

    # start email
    txt.append('summary=$(abed status | sed -e "s/\\x1b\\[.\\{1,5\\}m//g")')
    txt.append('echo -e "Job $PBS_JOBID started at `date`\\n\\n${summary}"'
            ' | mail $USER -s "Job $PBS_JOBID started"')
    txt.append('')

    # copy files to nodes
    cp_line = 'mpicopy ' + ' '.join(['${CURRENT}/' + x for x in 
        settings.PBS_MPICOPY])
    txt.append(cp_line)
    txt.append('')

    # calculate reduced runtime
    ttr = settings.PBS_WALLTIME * 60 - settings.PBS_TIME_REDUCE

    # run line
    txt.append('timeout %i mpiexec abed run' % ttr)
    txt.append('')

    # zip tasks
    txt.append('mkdir -p ${CURRENT}/bzips')
    txt.append('cd ${TMPDIR}/results')
    txt.append('')
    txt.append('for dset in `ls`')
    txt.append('do')
    txt.append('\tpacktime=$(date +%Y_%m_%d_%H_%M_%S)')
    txt.append('\ttar -c ${dset} | '
            'pbzip2 -vc -p14 -m1000 > ${packtime}_results_${dset}.tar.bz2')
    txt.append('\tcp ${packtime}_results_${dset}.tar.bz2 ${CURRENT}/bzips/')
    txt.append('done')
    txt.append('')

    # end email
    txt.append('echo "Job $PBS_JOBID finished at `date`" | mail $USER -s '
            '"Job $PBS_JOBID finished"')
    txt.append('')
    return '\n'.join(txt)



"""

Common HTML shared between all pages (the navbar for instance)

"""

import dominate

tags = dominate.tags

from .utils import AbedHTMLTypes
from ..utils import clean_str

def navbar_content(active):
    nav = tags.ul(_class='nav navbar-nav')
    for html_type in AbedHTMLTypes.types:
        curli = tags.li(_class='active') if html_type == active else tags.li()
        curli += tags.a(html_type[0], href=html_type[1])
        nav += curli
    return nav

def navbar(active=AbedHTMLTypes.INDEX):
    with tags.nav(_class='navbar navbar-inverse navbar-fixed-top') as bar:
        with tags.div(_class='container'):
            with tags.div(_class='navbar-header'):
                with tags.button(type='button',
                        _class='navbar-toggle collapsed',
                        data_toggle='collapse', data_target='#navbar',
                        aria_expanded='false', aria_controls='navbar'):
                    tags.span('Toggle navigation', _class='sr-only')
                    tags.span(_class='icon-bar')
                    tags.span(_class='icon-bar')
                    tags.span(_class='icon-bar')
                tags.a('ABED', _class='navbar-brand', href='#')
            with tags.nav(_class='collapse navbar-collapse', id='navbar'):
                navbar_content(active)
            tags.comment('/.nav-collapse')
    return bar

def generate_buttons(tables, attribute):
    values = sorted(set([getattr(t, attribute) for t in tables]))
    buttons = []
    for value in values:
        clean = clean_str(value)
        btn = {
                '_id': '%s_%s' % (attribute, clean),
                'name': attribute,
                'value': clean,
                'label': value,
                'active': False
                }
        buttons.append(btn)
    if buttons:
        buttons[0]['active'] = True
    return buttons

def bootstrap_radio_btn(_id, name, value, label, active=False):
    cls = 'btn btn-primary active' if active else 'btn btn-primary'
    btn = tags.label(_class=cls)
    if active:
        btn += tags.input(type='radio', id=_id, name=name, value=value, 
                checked=True)
    else:
        btn += tags.input(type='radio', id=_id, name=name, value=value)
    btn += dominate.util.text(label)
    return btn

def bootstrap_radio_group(btndefs):
    with tags.div(_class='btn-group', data_toggle='buttons') as bgrp:
        for btndef in btndefs:
            bootstrap_radio_btn(**btndef)
    return bgrp




"""
General functions for generating tables for Abed results

"""

from .models import AbedTable, AbedTableTypes
from .ranks import make_rank_table
from ..conf import settings

def make_tables_scalar(abed_cache, scalar):
    # First create the normal table
    table = build_tables_scalar(abed_cache, scalar)
    table.higher_better = (True if settings.SCALARS[scalar]['best'] == max else 
            False)
    table.type = AbedTableTypes.VALUES
    table.desc = 'Scalar: %s' % scalar
    table.name = scalar
    table.target = scalar
    table.is_metric = False
    # Now create the rank table from the generated table
    ranktable = make_rank_table(table)
    return [table, ranktable]

def build_tables_scalar(abed_cache, scalarname):
    table = AbedTable()
    table.headers = ['ID'] + sorted(abed_cache.methods)
    for i, dset in enumerate(sorted(abed_cache.datasets)):
        row = []
        for j, method in enumerate(sorted(abed_cache.methods)):
            values = list(abed_cache.get_scalar_values_dm(dset, method, 
                scalarname))
            best_value = settings.SCALARS[scalarname]['best'](values)
            rounded = round(best_value, settings.RESULT_PRECISION)
            fmt = '%%.%df' % settings.RESULT_PRECISION
            row.append(fmt % rounded)
        table.add_row(dset, row)
    return table



"""
Functions for managing tasks

"""

import os
import sys
import random

from itertools import product

from .conf import settings
from .exceptions import (AbedHashCollissionException, 
        AbedExperimentTypeException)
from .results.walk import walk_hashes
from .utils import error

def cartesian(params):
    return (dict(list(zip(params, x))) for x in product(*params.values()))

def check_size():
    if not sys.maxsize == 9223372036854775807:
        error("Running on a non 64-bit system. This may cause problems with "
                "hashes.")
        raise SystemExit

def task_hash(task):
    """
    This yields a hash of a list by combining the hashes of all list elements.
    """
    hsh = hash(frozenset(list(task.items())))
    hsh %= ((sys.maxsize + 1) * 2)
    return hsh

def init_tasks():
    if settings.TYPE == 'ASSESS':
        task_func = init_tasks_assess
    elif settings.TYPE == 'CV_TT':
        task_func = init_tasks_cv_tt
    elif settings.TYPE == 'RAW':
        task_func = init_tasks_raw

    try:
        return task_func()
    except AbedHashCollissionException:
        error("A hash collision occured. This rarely occurs naturally, so it"
                " is most likely caused by duplicate tasks in the task list. "
                "Abed does not currently support duplicate tasks.")
        raise SystemExit
    raise AbedExperimentTypeException

def init_tasks_assess():
    out = {}
    for dset in settings.DATASETS:
        for method in settings.METHODS:
            for prmset in cartesian(settings.PARAMS[method]):
                task = {key: value for key, value in prmset.items()}
                task['dataset'] = dset
                task['method'] = method
                hsh = task_hash(task)
                if hsh in out:
                    raise AbedHashCollissionException
                out[hsh] = task
    return out

def init_tasks_cv_tt():
    out = {}
    rng = random.Random(x=settings.CV_BASESEED)
    for train, test in settings.DATASETS:
        seed = rng.randint(0, 2**31-1)
        for method in settings.METHODS:
            for prmset in cartesian(settings.PARAMS[method]):
                task = {key: value for key, value in prmset.items()}
                task['train_dataset'] = train
                task['test_dataset'] = test
                task['method'] = method
                task['cv_seed'] = seed
                hsh = task_hash(task)
                if hsh in out:
                    raise AbedHashCollissionException
                out[hsh] = task
    return out

def init_tasks_raw():
    out = {}
    with open(settings.RAW_CMD_FILE, 'r') as fid:
        tasks = [x.strip() for x in fid.readlines() if x.strip()]
    for txttask in tasks:
        hsh = hash(txttask)
        hsh %= ((sys.maxsize + 1) * 2)
        if hsh in out:
            raise AbedHashCollissionException
        out[hsh] = txttask
    return out

def read_tasks():
    with open(settings.TASK_FILE, 'r') as fid:
        tasks = [l.strip() for l in fid.readlines() if l.strip()]
    tasks = list(map(int, tasks))
    grid = init_tasks()
    out = {}
    for key in tasks:
        out[key] = grid[key]
    return out

def update_tasks(tasks):
    delcnt = 0
    if not os.path.exists(settings.RESULT_DIR):
        return 0
    for hsh in walk_hashes():
        try:
            del tasks[int(hsh)]
            delcnt += 1
        except KeyError:
            pass
    return delcnt

def explain_tasks(all_tasks):
    for task in sorted(all_tasks.keys()):
        if settings.TYPE == 'RAW':
            cmd = all_tasks[task]
        else:
            d = {k:v for k, v in all_tasks[task].items()}
            command = settings.COMMANDS[d['method']]
            d['datadir'] = '{datadir}'
            d['execdir'] = '{execdir}'
            cmd = command.format(**d)
        print('%s : %s' % (task, cmd))



import os
import re

from setuptools import setup, find_packages

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

def get_data_files():
    datadir = os.path.join('share', 'data')
    datafiles = []
    for root, dirs, files in os.walk(datadir):
        if not dirs:
            newfiles = [os.path.join(root, f) for f in files]
            datafiles.append((root, newfiles))
    return datafiles

version = re.search("__version__ = '([^']+)'", 
        open('abed/__init__.py').read()).group(1)

setup(
        name = "abed",
        version = version,
        author = "Gertjan van den Burg",
        author_email = "gertjanvandenburg@gmail.com",
        description = ("A utility for Automated BEnchmark Distribution"),
        license = "GPL v2",
        packages = find_packages(),
        long_description = read('README.rst'),
        scripts = ['bin/abed'],
        install_requires = [
            'Fabric3==1.13.1.post1',
            'mpi4py==1.3.1',
            'tabulate',
            'bz2file',
            'progressbar',
            'dominate',
            'tqdm',
            'backports.lzma',
            'gitpython'
            ],
        data_files = get_data_files()
)



import dominate
import os

from .common import navbar
from .rank_graphs import get_table_id
from .utils import AbedHTMLTypes, copy_data_file
from ..conf import settings
from ..utils import info
from ..results.models import AbedTableTypes
from ..results.significance import global_difference, reference_difference

tags = dominate.tags

def create_rt_html(tables):
    html = generate_rt_html(tables)
    write_tables_html(html)

def generate_label_panel(lbl, lbl_table):
    headers = ['Method', 'Z-statistic', 'P-value', 'Sig. diff.']
    with tags.div(_class="panel panel-default") as panel:
        with tags.div(_class="panel-heading"):
            tags.h3('Target: %s' % lbl, _class='panel-title')
        with tags.div(_class='panel-body'):
            for table in lbl_table[lbl]:
                if len(settings.METHODS) <= 1:
                    continue
                tabid = get_table_id(table)
                if table.is_metric:
                    if settings.TYPE == 'ASSESS':
                        tags.p("Metric: %s" % table.metricname)
                    elif settings.TYPE == 'CV_TT':
                        with tags.div(_class="col-xs-6"):
                            tags.b("Train Metric on %s: %s" %
                                    (settings.YTRAIN_LABEL,
                                        table.trainmetricname))
                        with tags.div(_class="col-xs-6"):
                            tags.b("Test Metric on %s: %s" % (lbl,
                                table.testmetricname))
                with tags.div(id=tabid, _class="AbedRankTests"):
                    fval, fprob = global_difference(table)
                    tags.p("Friedman Test: F = %.4f with p-value %.6f" % 
                            (fval, fprob))
                    if settings.REFERENCE_METHOD is None:
                        continue
                    holms, CD = reference_difference(table)
                    tags.p("Holm's procedure, comparing all with %s:" % 
                            settings.REFERENCE_METHOD)
                    tags.p("Critical difference: %f" % CD)
                    with tags.table(id="rt_"+tabid, _class="display", 
                            cellspacing='0', width='100%'):
                        with tags.thead():
                            tr = tags.tr()
                            for hdr in headers:
                                tr += tags.th(hdr)
                        with tags.tbody():
                            for method, zscore, pval, sigdiff in holms:
                                tr = tags.tr()
                                tr += tags.td(method)
                                tr += tags.td(zscore)
                                tr += tags.td(pval)
                                tr += tags.td(str(int(sigdiff)))
                tags.hr()
    return panel

def generate_rt_html(tables):
    doc = dominate.document(title='ABED Rank Tests')

    with doc.head:
        # meta tags
        tags.meta(charset='utf-8')
        tags.meta(http_equiv='X-UA-Compatible', content='IE=edge')
        tags.meta(name='viewport',
                content='width=device-width, initial-scale=1')
        tags.meta(name='description', content='Rank graphs for ABED results')
        tags.meta(name='author', content='Gertjan van den Burg')

        # JQuery & Bootstrap JS
        tags.script(type='text/javascript',
                src=copy_data_file('jquery/1.11.3/jquery.min.js'))
        tags.script(type='text/javascript',
                src=copy_data_file('bootstrap-3.3.5-dist/js/bootstrap.min.js'))
        # Bootstrap core CSS
        tags.link(rel='stylesheet', href=copy_data_file(
            'bootstrap-3.3.5-dist/css/bootstrap.min.css'))

        # ABED CSS & JS
        tags.link(rel='stylesheet', href=copy_data_file('abed/css/abed.css'))

    tables = [t for t in tables if t.is_summary and t.type ==
            AbedTableTypes.RANKS]
    labels = sorted(set([t.target for t in tables]))
    lbl_table = {}
    for lbl in labels:
        lbl_table[lbl] = [t for t in tables if t.target == lbl]

    with doc:
        with tags.div(id='header'):
            navbar(active=AbedHTMLTypes.RANK_TESTS)
        with tags.div(id='content'):
            with tags.div(_class='container'):
                for lbl in sorted(lbl_table.keys()):
                    generate_label_panel(lbl, lbl_table)
            tags.comment('./container')
        with tags.div(id='footer'):
            pass

    return str(doc)

def write_tables_html(doc):
    fname = os.path.join(settings.OUTPUT_DIR, 'html',
            AbedHTMLTypes.RANK_TESTS[-1])
    with open(fname, 'w') as fid:
        fid.write(doc)
    info('Created output file: %s' % fname)



import dominate
import os

from .common import navbar, bootstrap_radio_group, generate_buttons
from .utils import AbedHTMLTypes, copy_data_file
from ..conf import settings
from ..utils import info, clean_str

tags = dominate.tags

def create_metric_tables_html(tables):
    html = generate_tables_html(tables)
    write_tables_html(html)

def get_table_id(table):
    values = []
    values.append(clean_str(table.target))
    if settings.TYPE == 'ASSESS':
        values.append(clean_str(table.metricname))
    elif settings.TYPE == 'CV_TT':
        values.append(clean_str(table.trainmetricname))
        values.append(clean_str(table.testmetricname))
    values.append(clean_str(table.type))
    if table.is_summary:
        values.append('summary')
    return '_'.join(values)

def generate_tables_html(tables):
    doc = dominate.document(title='ABED Metric Tables')

    with doc.head:
        # meta tags
        tags.meta(charset='utf-8')
        tags.meta(http_equiv='X-UA-Compatible', content='IE=edge')
        tags.meta(name='viewport',
                content='width=device-width, initial-scale=1')
        tags.meta(name='description', content='')
        tags.meta(name='author', content='')
        #tags.link(rel='icon', href=copy_data_file('ABED/images/favicon.ico'))

        # JQuery & Bootstrap JS
        tags.script(src=copy_data_file('jquery/1.11.3/jquery.min.js'))
        tags.script(
                src=copy_data_file('bootstrap-3.3.5-dist/js/bootstrap.min.js'))
        # Bootstrap core CSS
        tags.link(rel='stylesheet', href=copy_data_file(
            'bootstrap-3.3.5-dist/css/bootstrap.min.css'))

        # Datatables CSS & JS
        tags.link(rel='stylesheet', href=copy_data_file(
            'DataTables-1.10.7/media/css/jquery.dataTables.css'))
        tags.script(type='text/javascript', src=copy_data_file(
            'DataTables-1.10.7/media/js/jquery.dataTables.js'))

        # ABED CSS & JS
        tags.link(rel='stylesheet', href=copy_data_file('abed/css/abed.css'))
        tags.script(type="text/javascript",
                src=copy_data_file('abed/js/abed_showtables.js'))
        tags.script(type="text/javascript",
                src=copy_data_file('abed/js/abed_metrictables.js'))

    with doc:
        navbar(active=AbedHTMLTypes.METRIC_TABLES)
        with tags.div(_class='container'):
            with tags.table(id='AbedButtons', _class='display',
                    cellspacing='0', width='100%'):
                with tags.thead():
                    tr = tags.tr()
                    tr += tags.th('Target')
                    if settings.TYPE == 'ASSESS':
                        tr += tags.th('Metric')
                    elif settings.TYPE == 'CV_TT':
                        tr += tags.th('Train Metric (on %s)' %
                                settings.YTRAIN_LABEL)
                        tr += tags.th('Test Metric (on target)')
                    tr += tags.th('Output Type')
                with tags.tbody():
                    tr = tags.tr()
                    tr += tags.td(bootstrap_radio_group(generate_buttons(
                        tables, 'target')))
                    if settings.TYPE == 'ASSESS':
                        tr += tags.td(bootstrap_radio_group(generate_buttons(
                            tables, 'metricname')))
                    elif settings.TYPE == 'CV_TT':
                        tr += tags.td(bootstrap_radio_group(generate_buttons(
                            tables, 'trainmetricname')))
                        tr += tags.td(bootstrap_radio_group(generate_buttons(
                            tables, 'testmetricname')))
                    tr += tags.td(bootstrap_radio_group(generate_buttons(
                        tables, 'type')))
            for table in tables:
                tabid = get_table_id(table)
                with tags.div(id='div_'+ tabid, _class='AbedTable'):
                    if 'summary' in tabid:
                        tags.p("Summary Table:")
                    with tags.table(id='tbl_'+tabid, _class='display',
                            cellspacing='0', width='100%'):
                        with tags.thead():
                            tr = tags.tr()
                            for hdr in table.headers:
                                tr += tags.th(hdr)
                        if not 'summary' in tabid:
                            with tags.tfoot():
                                tr = tags.tr()
                                for hdr in table.headers:
                                    tr += tags.th(hdr)
        tags.comment('/.container')
    return str(doc)

def write_tables_html(doc):
    fname = '%s%s%s%s%s' % (settings.OUTPUT_DIR, os.sep, 'html', os.sep,
            'metric_tables.html')
    with open(fname, 'w') as fid:
        fid.write(doc)
    info("Created output file: %s" % fname)
